/*****************************************************************************
 * Copyright (C) 2004-2009 Christoph Thielecke <crissi99@gmx.de>             *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This package is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this package; if not, write to the Free Software               *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA *
 *****************************************************************************/

#ifndef CISCOCERTIFICATEENROLLMENT_H
#define CISCOCERTIFICATEENROLLMENT_H

#include <QProcess>
#include <Qt3Support/Q3Wizard>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QTimer>

#include <kdialog.h>
#include <kprogressdialog.h>

#include "ciscocertificateenrollmentwidget.h"
#include "ciscocertificateenrollmentfinishwidget.h"
#include "ciscocertificateenrollmentdatadialog.h"
#include "kvpncconfig.h"

class CiscoCertificateEnrollment : public Q3Wizard
{
    Q_OBJECT

public:
    CiscoCertificateEnrollment(QWidget *parent, const char* caption, KVpncConfig *GlobalConfig);
    ~CiscoCertificateEnrollment();

    QProcess *EnrollmentProcess;
    QProcess *DeleteProcess;
    KVpncConfig *GlobalConfig;
    bool done;
    bool finished;
    bool valuesOk;
    QStringList *env;

public slots:
    void accept();
    void canAccept();
    void enrollmentToggled(bool);
    void setupPages();
    void showPage(QWidget *page);
    void next();
    void back();
    void reject();
    void readFromStdout();
    void readFromStderr();
    void readFromStdout_cancel();
    void readFromStderr_cancel();
    void checkProgress();
    void enrollmentProcessFinished(int, QProcess::ExitStatus);
    void cancelProcessFinished(int, QProcess::ExitStatus);

public:
    CiscoCertificateEnrollmentWidget *selectionpage;
    CiscoCertificateEnrollmentDataDialog *datapage;
    CiscoCertificateEnrollmentFinishWidget *finishpage;
    QWidget* currentpage; // pointer to current page
    QWidget* previouspage; //< pointer to previous page
    KProgressDialog *ProgressDlg;

public:
    QString type;
    QString successmsg;
    int success;
    QString CA;
    QString CaUrl;
    QString ChallengePassword;
    QString CaDomain;
    QString FileEncoding;
    QString FilenameURL;
    QString Ip;
    QString State;
    QString Department;
    QString Company;
    QString Domain;
    QString Name;
    QString Email;
    QString Country;

private:
    QTimer CheckEnrollmentTimer;
};

#endif
